@@ -10,6 +10,7 @@ from keras_cv_attention_models.attention_layers import (
     drop_block,
     drop_connect_rates_split,
     layer_norm,
+    PositionalEmbedding,
     add_pre_post_process,
 )
 from keras_cv_attention_models.download_and_load import reload_model_weights
@@ -18,9 +19,18 @@ LAYER_NORM_EPSILON = 1e-6
 
 PRETRAINED_DICT = {
     "beit_base_patch16": {"imagenet21k-ft1k": {224: "d7102337a13a3983f3b6470de77b5d5c", 384: "76353026477c60f8fdcbcc749fea17b3"}},
+    "beit_v2_base_patch16": {"imagenet21k-ft1k": {224: "d001dcb67cdda16bfdbb2873ab9b13c8"}},
     "beit_large_patch16": {
         "imagenet21k-ft1k": {224: "fce2d162e7fa4dba9a1b1fc5e1dec5ce", 384: "158934d07dd8b1e1c6b96883aa00a748", 512: "64d18088e91df243960e5830aab80a6e"}
     },
+    "beit_v2_large_patch16": {"imagenet21k-ft1k": {224: "b3cee12a545bfb676f9f426ee7158d27"}},
+    "eva_giant_patch14": {
+        "imagenet21k-ft1k": {224: "5a475db6696d6e36ea896ec5dbd1c20d", 336: "fd8eeec10d6b6cb607ce033ea85b8e80", 560: "0ef0d2961523fb2047fbdb59cc347c17"}
+    },
+    "eva_large_patch14": {"imagenet21k-ft1k": {196: "bbeea886fbde4bd1c8c9876345273a99", 336: "4928faafd0177fe8f0d02dab4abc8e83"}},
+    "flexivit_small": {"imagenet": {240: "efb73a97d099a491b69ebfaf8a337df8"}},
+    "flexivit_base": {"imagenet": {240: "dac627debb194928db01e1b9b7a548fd"}},
+    "flexivit_large": {"imagenet": {240: "6faa953227d2ef1df6758f8eb7234490"}},
 }
 
 
@@ -128,13 +138,18 @@ class MultiHeadRelativePositionalEmbedding(keras.layers.Layer):
         return fig
 
 
-def attention_block(inputs, num_heads=4, key_dim=0, out_weight=True, out_bias=False, qv_bias=True, attn_height=-1, attn_dropout=0, name=None):
+def attention_block(
+    inputs, num_heads=4, key_dim=0, qv_bias=True, qkv_bias=False, out_weight=True, out_bias=False, use_pos_emb=False, attn_height=-1, attn_dropout=0, name=None
+):
     _, bb, cc = inputs.shape
     key_dim = key_dim if key_dim > 0 else cc // num_heads
     qk_scale = float(1.0 / tf.math.sqrt(tf.cast(key_dim, "float32")))
     emded_dim = num_heads * key_dim
 
-    qkv = keras.layers.Dense(emded_dim * 3, use_bias=False, name=name and name + "qkv")(inputs)
+    # if qkv_bias, just use bias in qkv_dense, and set qv_bias False
+    qkv_bias, qv_bias = (True, False) if qkv_bias else (False, qv_bias)
+
+    qkv = keras.layers.Dense(emded_dim * 3, use_bias=qkv_bias, name=name and name + "qkv")(inputs)
     qkv = tf.reshape(qkv, [-1, bb, qkv.shape[-1]])
     query, key, value = tf.split(qkv, 3, axis=-1)
     # query = [batch, num_heads, cls_token + hh * ww, key_dim]
@@ -153,7 +168,8 @@ def attention_block(inputs, num_heads=4, key_dim=0, out_weight=True, out_bias=Fa
     query *= qk_scale
     # [batch, num_heads, cls_token + hh * ww, cls_token + hh * ww]
     attention_scores = keras.layers.Lambda(lambda xx: tf.matmul(xx[0], xx[1]))([query, key])
-    attention_scores = MultiHeadRelativePositionalEmbedding(attn_height=attn_height, name=name and name + "pos_emb")(attention_scores)
+    if use_pos_emb:
+        attention_scores = MultiHeadRelativePositionalEmbedding(attn_height=attn_height, name=name and name + "pos_emb")(attention_scores)
     # attention_scores = tf.nn.softmax(attention_scores, axis=-1, name=name and name + "_attention_scores")
     attention_scores = keras.layers.Softmax(axis=-1, name=name and name + "attention_scores")(attention_scores)
 
@@ -176,7 +192,7 @@ def attention_mlp_block(inputs, embed_dim, gamma_init_value=0.1, mlp_ratio=4, dr
     # print(f">>>> {drop_rate = }")
     nn = layer_norm(inputs, epsilon=LAYER_NORM_EPSILON, name=name + "attn_")
     nn = attention_block(nn, **attn_params, name=name + "attn_")
-    nn = ChannelAffine(use_bias=False, weight_init_value=gamma_init_value, name=name + "attn_gamma")(nn)
+    nn = ChannelAffine(use_bias=False, weight_init_value=gamma_init_value, name=name + "attn_gamma")(nn) if gamma_init_value > 0 else nn
     nn = drop_block(nn, drop_rate)
     attn_out = keras.layers.Add(name=name + "attn_out")([inputs, nn])
 
@@ -185,7 +201,7 @@ def attention_mlp_block(inputs, embed_dim, gamma_init_value=0.1, mlp_ratio=4, dr
     nn = keras.layers.Dense(embed_dim * mlp_ratio, name=name + "mlp_dense_1")(nn)
     nn = activation_by_name(nn, activation, name=name + "mlp_" + activation)
     nn = keras.layers.Dense(embed_dim, name=name + "mlp_dense_2")(nn)
-    nn = ChannelAffine(use_bias=False, weight_init_value=gamma_init_value, name=name + "mlp_gamma")(nn)
+    nn = ChannelAffine(use_bias=False, weight_init_value=gamma_init_value, name=name + "mlp_gamma")(nn) if gamma_init_value > 0 else nn
     nn = drop_block(nn, drop_rate)
     nn = keras.layers.Add(name=name + "mlp_output")([attn_out, nn])
     return nn
@@ -206,6 +222,60 @@ class HeadInitializer(tf.initializers.Initializer):
         return base_config
 
 
+@tf.keras.utils.register_keras_serializable(package="beit")
+class PatchConv2DWithResampleWeights(keras.layers.Conv2D):
+    def __init__(self, filters, kernel_size=1, strides=1, padding="valid", use_bias=True, groups=1, **kwargs):
+        super().__init__(filters, kernel_size=kernel_size, strides=strides, padding="valid", use_bias=use_bias, groups=groups, **kwargs)
+        self.padding = padding
+
+    def build(self, input_shape):
+        pad = (self.kernel_size[0] // 2, self.kernel_size[1] // 2)
+        if self.padding.upper() == "SAME" and max(pad) != 0:
+            self.pad = [[0, 0], [pad[0], pad[0]], [pad[1], pad[1]], [0, 0]]
+        else:
+            self.pad = None
+        super().build(input_shape)
+
+    def call(self, inputs, **kwargs):
+        if self.pad is not None:
+            inputs = tf.pad(inputs, self.pad)
+        return super().call(inputs)
+
+    def load_resized_weights(self, source_layer, method="bilinear"):
+        import numpy as np
+
+        if isinstance(source_layer, dict):
+            source_kernel, source_bias = source_layer["kernel:0"], source_layer["bias:0"]  # weights
+        else:
+            source_kernel, source_bias = source_layer.kernel, source_layer.bias  # layer
+
+        # From FlexiViT https://github.com/google-research/big_vision/blob/main/big_vision/models/proj/flexi/vit.py#L30
+        # Paper [PDF 2212.08013 FlexiViT: One Model for All Patch Sizes](https://arxiv.org/pdf/2212.08013.pdf)
+
+        # assume it's channel_last format, source_kernel shape `[patch_size, patch_size, in_channel, out_channel]`
+        source_kernel = np.array(source_kernel)
+        source_shape, target_shape = source_kernel.shape[:2], self.kernel_size
+
+        # get_resize_mat(old_shape, target_shape)
+        # NOTE: we are using tf.image.resize here to match the resize operations in
+        # the data preprocessing pipeline.
+        mat = []
+        for idx in range(source_shape[0] * source_shape[1]):
+            basis_vec = np.zeros(source_shape)
+            basis_vec[np.unravel_index(idx, source_shape)] = 1.0
+            vec = tf.image.resize(np.expand_dims(basis_vec, -1), target_shape, method=method).numpy().reshape(-1)
+            mat.append(vec)
+        resize_mat_pinv = np.linalg.pinv(np.stack(mat))
+
+        # v_resample_kernel = jax.vmap(jax.vmap(lambda kernel: (resize_mat_pinv @ kernel.reshape(-1)).reshape(new_hw), 2, 2), 3, 3)
+        # cc = v_resample_kernel(old)
+        # As it's only one weight, just using two loop here, instead of `jax.vmap`
+        target_weights = np.stack([[(resize_mat_pinv @ jj.reshape(-1)).reshape(target_shape) for jj in ii] for ii in source_kernel.transpose([3, 2, 0, 1])])
+        target_weights = target_weights.transpose([2, 3, 1, 0])
+        self.kernel.assign(target_weights)
+        self.bias.assign(source_bias)
+
+
 def Beit(
     depth=12,
     embed_dim=768,
@@ -213,37 +283,52 @@ def Beit(
     mlp_ratio=4,
     patch_size=16,
     attn_key_dim=0,
-    attn_qv_bias=True,
+    attn_qv_bias=True,  # Default False for Vit, True for Beit, if True and attn_qkv_bias being False, will add BiasLayer for query and key.
+    attn_qkv_bias=False,  # True for Vit, False for Beit, if True, will just use bias in qkv_dense, and set qv_bias False.
     attn_out_weight=True,
     attn_out_bias=True,
     attn_dropout=0,
-    gamma_init_value=0.1,
+    gamma_init_value=0.1,  # 0 for Vit, 0.1 for Beit, if > 0 will use `layer_scale` on block output
+    use_abs_pos_emb=False,  # True for Vit, False for Beit, whether use abcolute positional embedding or relative one in attention blocks
+    use_abs_pos_emb_on_cls_token=True,  # False for FlexiViT, no_embed_class in timm. If use_abs_pos_emb is True, whether apply pos_emb on cls_token.
+    use_mean_pooling=True,  # False for Vit, True for Beit, whether use use mean output or `class_token` output
     input_shape=(224, 224, 3),
     num_classes=1000,
     activation="gelu",
     drop_connect_rate=0,
-    use_mean_pooling=True,
     classifier_activation="softmax",
-    pretrained="imagenet21k-ft1k",
+    pretrained=None,
+    force_reload_mismatch=False,  # set True if patch_size changed, will force reloading pos_emb and stem_conv weights
     model_name="beit",
     kwargs=None,
 ):
     inputs = keras.layers.Input(input_shape)
 
     """ forward_embeddings """
-    nn = conv2d_no_bias(inputs, embed_dim, patch_size, strides=patch_size, padding="valid", use_bias=True, name="stem_")
+    # nn = conv2d_no_bias(inputs, embed_dim, patch_size, strides=patch_size, padding="valid", use_bias=True, name="stem_")
+    nn = PatchConv2DWithResampleWeights(embed_dim, patch_size, strides=patch_size, padding="valid", use_bias=True, name="stem_conv")(inputs)
     patch_height = nn.shape[1]
     nn = keras.layers.Reshape([-1, nn.shape[-1]])(nn)
-    nn = ClassToken(name="cls_token")(nn)
+
+    if use_abs_pos_emb and use_abs_pos_emb_on_cls_token:  # EvaLarge and EvaGiant
+        nn = ClassToken(name="cls_token")(nn)
+        nn = PositionalEmbedding(input_height=patch_height, name="positional_embedding")(nn)
+    elif use_abs_pos_emb:  # FlexiViT models
+        nn = PositionalEmbedding(input_height=patch_height, name="positional_embedding")(nn)
+        nn = ClassToken(name="cls_token")(nn)
+    else:  # Beit and BeitV2
+        nn = ClassToken(name="cls_token")(nn)
 
     attn_params = {
         "num_heads": num_heads,
         "key_dim": attn_key_dim,
         "qv_bias": attn_qv_bias,
+        "qkv_bias": attn_qkv_bias,
         "out_weight": attn_out_weight,
         "out_bias": attn_out_bias,
         "attn_height": patch_height,
         "attn_dropout": attn_dropout,
+        "use_pos_emb": not use_abs_pos_emb,
     }
 
     """ forward_tokens """
@@ -266,7 +351,8 @@ def Beit(
         )(nn)
     model = tf.keras.models.Model(inputs, nn, name=model_name)
     add_pre_post_process(model, rescale_mode="tf")
-    reload_model_weights(model, PRETRAINED_DICT, "beit", pretrained, MultiHeadRelativePositionalEmbedding)
+    mismatch_class = [PatchConv2DWithResampleWeights, PositionalEmbedding if use_abs_pos_emb else MultiHeadRelativePositionalEmbedding]
+    reload_model_weights(model, PRETRAINED_DICT, "beit", pretrained, mismatch_class, force_reload_mismatch)
     return model
 
 
@@ -275,7 +361,13 @@ def BeitBasePatch16(input_shape=(224, 224, 3), num_classes=1000, activation="gel
     depth = 12
     num_heads = 12
     gamma_init_value = 0.1
-    return Beit(**locals(), model_name="beit_base_patch16", **kwargs)
+    force_reload_mismatch = kwargs.get("patch_size", 16) != 16  # If patch_size not 16, force reload pos_emb and stem_conv weights
+    kwargs.pop("kwargs", None)  # From BeitV2BasePatch16
+    return Beit(**locals(), model_name=kwargs.pop("model_name", "beit_base_patch16"), **kwargs)
+
+
+def BeitV2BasePatch16(input_shape=(224, 224, 3), num_classes=1000, activation="gelu", classifier_activation="softmax", pretrained="imagenet21k-ft1k", **kwargs):
+    return BeitBasePatch16(**locals(), **kwargs, model_name="beit_v2_base_patch16")
 
 
 def BeitLargePatch16(input_shape=(224, 224, 3), num_classes=1000, activation="gelu", classifier_activation="softmax", pretrained="imagenet21k-ft1k", **kwargs):
@@ -283,4 +375,40 @@ def BeitLargePatch16(input_shape=(224, 224, 3), num_classes=1000, activation="ge
     depth = 24
     num_heads = 16
     gamma_init_value = 1e-5
-    return Beit(**locals(), model_name="beit_large_patch16", **kwargs)
+    force_reload_mismatch = kwargs.get("patch_size", 16) != 16  # If patch_size not 16, force reload pos_emb and stem_conv weights
+    kwargs.pop("kwargs", None)  # From BeitV2LargePatch16
+    return Beit(**locals(), model_name=kwargs.pop("model_name", "beit_large_patch16"), **kwargs)
+
+
+def BeitV2LargePatch16(
+    input_shape=(224, 224, 3), num_classes=1000, activation="gelu", classifier_activation="softmax", pretrained="imagenet21k-ft1k", **kwargs
+):
+    return BeitLargePatch16(**locals(), **kwargs, model_name="beit_v2_large_patch16")
+
+
+""" keras_model_load_weights_from_pytorch_model """
+
+
+def keras_model_load_weights_from_pytorch_model(keras_model, timm_vit_model, save_name=None):
+    from keras_cv_attention_models import download_and_load, attention_layers
+
+    skip_weights = ["relative_position_index"]
+    unstack_weights = ["cls_token", "gamma_1", "gamma_2", "relative_position_bias_table", "q_bias", "v_bias", "pos_embed"]
+    tail_align_dict = {"attn_gamma": -6, "mlp_gamma": -9, "attn_query_bias": -1, "attn_value_bias": -1, "attn_pos_emb": -1}
+    full_name_align_dict = {"cls_token": -2 if "flexivit" in keras_model.name else -1, "positional_embedding": -1}
+    additional_transfer = {attention_layers.MultiHeadRelativePositionalEmbedding: lambda ww: [ww[0].T]}
+
+    download_and_load.keras_reload_from_torch_model(
+        torch_model=timm_vit_model,
+        keras_model=keras_model,
+        input_shape=keras_model.input_shape[1:-1],
+        skip_weights=skip_weights,
+        unstack_weights=unstack_weights,
+        tail_align_dict=tail_align_dict,
+        full_name_align_dict=full_name_align_dict,
+        tail_split_position=1,
+        additional_transfer=additional_transfer,
+        save_name=save_name if save_name is not None else (keras_model.name + "_{}.h5".format(keras_model.input_shape[1])),
+        do_convert=True,
+        # do_predict=False if "eva_giant" in keras_model.name else True,
+    )
