@@ -1,11 +1,11 @@
 import os
 import tensorflow as tf
-import tensorflow_datasets as tfds
 from keras_cv_attention_models.coco import anchors_func, data
 from tqdm import tqdm
 
 
-class DecodePredictions:
+@tf.keras.utils.register_keras_serializable(package="kecam/coco")
+class DecodePredictions(tf.keras.layers.Layer):
     """
     The most simple version decoding prediction and NMS:
 
@@ -26,8 +26,25 @@ class DecodePredictions:
     """
 
     def __init__(
-        self, input_shape=512, pyramid_levels=[3, 7], anchors_mode=None, use_object_scores="auto", anchor_scale="auto", aspect_ratios=(1, 2, 0.5), num_scales=3
+        self,
+        input_shape=512,
+        pyramid_levels=[3, 7],
+        anchors_mode=None,
+        use_object_scores="auto",
+        anchor_scale="auto",
+        aspect_ratios=(1, 2, 0.5),
+        num_scales=3,
+        score_threshold=0.3,  # decode parameter, can be set new value in `self.call`
+        iou_or_sigma=0.5,  # decode parameter, can be set new value in `self.call`
+        max_output_size=100,  # decode parameter, can be set new value in `self.call`
+        method="hard",  # decode parameter, can be set new value in `self.call`
+        mode="global",  # decode parameter, can be set new value in `self.call`
+        topk=0,  # decode parameter, can be set new value in `self.call`
+        use_static_output=False,  # Set to True if using this as an actual layer, especially for converting tflite
+        **kwargs,
     ):
+        super().__init__(**kwargs)
+
         self.pyramid_levels = list(range(min(pyramid_levels), max(pyramid_levels) + 1))
         use_object_scores, num_anchors, anchor_scale = anchors_func.get_anchors_mode_parameters(anchors_mode, use_object_scores, "auto", anchor_scale)
         self.aspect_ratios, self.num_scales = aspect_ratios, num_scales
@@ -36,6 +53,17 @@ class DecodePredictions:
             self.__init_anchor__(input_shape)
         else:
             self.anchors = None
+        self.__input_shape__ = input_shape
+        self.use_static_output = use_static_output
+        self.nms_kwargs = {
+            "score_threshold": score_threshold,
+            "iou_or_sigma": iou_or_sigma,
+            "max_output_size": max_output_size,
+            "method": method,
+            "mode": mode,
+            "topk": topk,
+        }
+        super().build(input_shape)
 
     def __init_anchor__(self, input_shape):
         input_shape = input_shape[:2] if isinstance(input_shape, (list, tuple)) else (input_shape, input_shape)
@@ -46,6 +74,7 @@ class DecodePredictions:
         else:
             grid_zero_start = False
             self.anchors = anchors_func.get_anchors(input_shape, self.pyramid_levels, self.aspect_ratios, self.num_scales, self.anchor_scale, grid_zero_start)
+        self.__input_shape__ = input_shape
         return self.anchors
 
     def __topk_class_boxes_single__(self, pred, topk=5000):
@@ -86,17 +115,25 @@ class DecodePredictions:
         cls_offset = tf.cast(labels, bbs.dtype) * (tf.reduce_max(bbs) + 1)
         bbs_per_class = bbs + tf.expand_dims(cls_offset, -1)
         rr, nms_scores = tf.image.non_max_suppression_with_scores(bbs_per_class, ccs, max_output_size, iou_threshold, score_threshold, soft_nms_sigma)
-        return tf.gather(bbs, rr).numpy(), tf.gather(labels, rr).numpy(), nms_scores.numpy()
+        return tf.gather(bbs, rr), tf.gather(labels, rr), nms_scores
 
     def __nms_global__(self, bbs, ccs, labels, score_threshold=0.3, iou_threshold=0.5, soft_nms_sigma=0.5, max_output_size=100):
         rr, nms_scores = tf.image.non_max_suppression_with_scores(bbs, ccs, max_output_size, iou_threshold, score_threshold, soft_nms_sigma)
-        return tf.gather(bbs, rr).numpy(), tf.gather(labels, rr).numpy(), nms_scores.numpy()
+        return tf.gather(bbs, rr), tf.gather(labels, rr), nms_scores
 
     def __object_score_split__(self, pred):
         return pred[:, :-1], pred[:, -1]  # May overwrite
 
+    def __to_static__(self, bboxs, lables, confidences, max_output_size=100):
+        indices = tf.expand_dims(tf.range(tf.shape(bboxs)[0]), -1)
+        lables = tf.cast(lables, bboxs.dtype)
+        concated = tf.concat([bboxs, tf.expand_dims(lables, -1), tf.expand_dims(confidences, -1)], axis=-1)
+        concated = tf.tensor_scatter_nd_update(tf.zeros([max_output_size, concated.shape[-1]], dtype=bboxs.dtype), indices, concated)
+        return concated
+
     def __decode_single__(self, pred, score_threshold=0.3, iou_or_sigma=0.5, max_output_size=100, method="hard", mode="global", topk=0, input_shape=None):
         # https://github.com/google/automl/tree/master/efficientdet/tf2/postprocess.py#L159
+        pred = tf.cast(pred, tf.float32)
         if input_shape is not None:
             self.__init_anchor__(input_shape)
 
@@ -118,20 +155,50 @@ class DecodePredictions:
         iou_threshold, soft_nms_sigma = (1.0, iou_or_sigma / 2) if method.lower() == "gaussian" else (iou_or_sigma, 0.0)
 
         if mode == "per_class":
-            return self.__nms_per_class__(bbs_decoded, ccs, labels, score_threshold, iou_threshold, soft_nms_sigma, max_output_size)
+            bboxs, lables, confidences = self.__nms_per_class__(bbs_decoded, ccs, labels, score_threshold, iou_threshold, soft_nms_sigma, max_output_size)
         else:
-            return self.__nms_global__(bbs_decoded, ccs, labels, score_threshold, iou_threshold, soft_nms_sigma, max_output_size)
+            bboxs, lables, confidences = self.__nms_global__(bbs_decoded, ccs, labels, score_threshold, iou_threshold, soft_nms_sigma, max_output_size)
 
-    def __call__(self, preds, score_threshold=0.3, iou_or_sigma=0.5, max_output_size=100, method="hard", mode="global", topk=0, input_shape=None):
+        return self.__to_static__(bboxs, lables, confidences, max_output_size) if self.use_static_output else (bboxs, lables, confidences)
+
+    def call(self, preds, input_shape=None, training=False, **nms_kwargs):
         """
         https://github.com/google/automl/tree/master/efficientdet/tf2/postprocess.py#L159
-        iou_or_sigma: means `soft_nms_sigma` if method is "gaussian", else `iou_threshold`.
-        method: "gaussian" or "hard".
-        mode: "global" or "per_class". "per_class" is strategy from `torchvision.ops.batched_nms`
-        topk: Using topk highest scores, each bbox may have multi labels. Set `0` to disable, `-1` using all.
+
+        input_shape: actual input shape if model using dynamic input shape `[None, None, 3]`.
+        nms_kwargs:
+          score_threshold: float value in (0, 1), min score threshold, lower output score will be excluded. Default 0.3.
+          iou_or_sigma: means `soft_nms_sigma` if method is "gaussian", else `iou_threshold`. Default 0.5.
+          max_output_size: max output size for `tf.image.non_max_suppression_with_scores`. Default 100.
+              If use_static_output=True, fixed output shape will be `[batch, max_output_size, 6]`.
+          method: "gaussian" or "hard".  Default "hard".
+          mode: "global" or "per_class". "per_class" is strategy from `torchvision.ops.batched_nms`. Default "global".
+          topk: Using topk highest scores, each bbox may have multi labels. Set `0` to disable, `-1` using all. Default 0.
         """
-        preds = preds if len(preds.shape) == 3 else [preds]
-        return [self.__decode_single__(pred, score_threshold, iou_or_sigma, max_output_size, method, mode, topk, input_shape) for pred in preds]
+        self.nms_kwargs.update(nms_kwargs)
+        if self.use_static_output:
+            return tf.map_fn(lambda xx: self.__decode_single__(xx, **nms_kwargs), preds)
+        elif len(preds.shape) == 3:
+            return [self.__decode_single__(pred, **self.nms_kwargs, input_shape=input_shape) for pred in preds]
+        else:
+            return self.__decode_single__(preds, **self.nms_kwargs, input_shape=input_shape)
+
+    def get_config(self):
+        config = super().get_config()
+        config.update(
+            {
+                "input_shape": self.__input_shape__,
+                "pyramid_levels": self.pyramid_levels,
+                "anchors_mode": self.anchors_mode,
+                "use_object_scores": self.use_object_scores,
+                "anchor_scale": self.anchor_scale,
+                "aspect_ratios": self.aspect_ratios,
+                "num_scales": self.num_scales,
+                "use_static_output": self.use_static_output,
+            }
+        )
+        config.update(self.nms_kwargs)
+        return config
 
 
 def scale_bboxes_back_single(bboxes, image_shape, scale, pad_top, pad_left, target_shape):
@@ -170,6 +237,8 @@ def init_eval_dataset(
     letterbox_pad=-1,
     use_bgr_input=False,
 ):
+    import tensorflow_datasets as tfds
+
     dataset = data.detection_dataset_from_custom_json(data_name) if data_name.endswith(".json") else tfds.load(data_name)
     ds = dataset.get("validation", dataset.get("test", None))
 
@@ -201,7 +270,7 @@ def model_detection_and_decode(model, eval_dataset, pred_decoder, nms_kwargs={},
         # Loop on batch
         for rr, image_shape, scale, pad_top, pad_left, image_id in zip(decoded_preds, original_image_shapes, scales, pad_tops, pad_lefts, image_ids):
             bboxes, labels, scores = rr
-            image_id = image_id.numpy()
+            image_id, bboxes, labels, scores = image_id.numpy(), bboxes.numpy(), labels.numpy(), scores.numpy()
             if image_id_map is not None:
                 image_id = image_id_map[image_id.decode() if isinstance(image_id, bytes) else image_id]
             bboxes = scale_bboxes_back_single(bboxes, image_shape, scale, pad_top, pad_left, target_shape).numpy()
@@ -318,7 +387,7 @@ class COCOEvalCallback(tf.keras.callbacks.Callback):
         model_basic_save_name=None,
     ):
         super().__init__()
-        self.anchors_mode, self.anchor_scale = anchors_mode, anchor_scale
+        self.anchors_mode = anchors_mode
         self.take_samples, self.annotation_file, self.start_epoch, self.frequency = take_samples, annotation_file, start_epoch, frequency
         self.save_json, self.model_basic_save_name, self.save_path, self.item_key = save_json, model_basic_save_name, "checkpoints", "val_ap_ar"
         self.data_name = data_name
@@ -340,6 +409,12 @@ class COCOEvalCallback(tf.keras.callbacks.Callback):
             "mode": nms_mode,
             "topk": nms_topk,
         }
+        self.anchor_kwargs = {
+            "anchor_scale": anchor_scale,
+            "aspect_ratios": aspect_ratios,
+            "num_scales": num_scales,
+        }
+        self.efficient_det_num_anchors = len(aspect_ratios) * num_scales
 
         self.is_coco = True if data_name.startswith("coco") and not data_name.endswith(".json") else False
         if self.data_name.endswith(".json") and self.annotation_file is None:
@@ -354,6 +429,8 @@ class COCOEvalCallback(tf.keras.callbacks.Callback):
         self.eval_dataset = init_eval_dataset(input_shape=input_shape, **self.dataset_kwargs)
         if self.anchors_mode is None or self.anchors_mode == "auto":
             self.anchors_mode, num_anchors = anchors_func.get_anchors_mode_by_anchors(input_shape, total_anchors=output_shape[1])
+        elif self.anchors_mode == anchors_func.EFFICIENTDET_MODE:
+            num_anchors = self.efficient_det_num_anchors
         else:
             num_anchors = anchors_func.NUM_ANCHORS.get(self.anchors_mode, 9)
         pyramid_levels = anchors_func.get_pyramid_levels_by_anchors(input_shape, total_anchors=output_shape[1], num_anchors=num_anchors)
@@ -361,9 +438,7 @@ class COCOEvalCallback(tf.keras.callbacks.Callback):
         # print(">>>>", self.dataset_kwargs)
         # print(">>>>", self.nms_kwargs)
 
-        self.pred_decoder = DecodePredictions(
-            input_shape, pyramid_levels, self.anchors_mode, anchor_scale=self.anchor_scale, aspect_ratios=aspect_ratios, num_scales=num_scales
-        )
+        self.pred_decoder = DecodePredictions(input_shape, pyramid_levels, self.anchors_mode, **self.anchor_kwargs)
 
         # Training saving best
         if self.model_basic_save_name is not None:
@@ -381,7 +456,7 @@ class COCOEvalCallback(tf.keras.callbacks.Callback):
                 self.dataset_kwargs["rescale_mode"] = getattr(self.model, "rescale_mode", "torch")
             self.build(self.model.input_shape, self.model.output_shape)
 
-        if epoch < self.start_epoch or epoch % self.frequency != 0:
+        if epoch + 1 < self.start_epoch or epoch % self.frequency != 0:
             return
 
         # pred_decoder = self.model.decode_predictions
