@@ -5,6 +5,10 @@ import tensorflow as tf
 from keras_cv_attention_models.imagenet import data
 from keras_cv_attention_models.model_surgery import change_model_input_shape
 
+CLASS_INDEX_IMAGENET21K = None
+
+""" Imagenet evaluation """
+
 
 class TorchModelInterf:
     def __init__(self, model):
@@ -93,6 +97,7 @@ class ONNXModelInterf:
         if self.data_format == "NCHW":
             imgs = imgs.transpose(0, 3, 1, 2)
         return self.ort_session.run(self.output_names, {self.input_name: imgs.astype("float32")})[0]
+        # return np.array([self.ort_session.run(self.output_names, {self.input_name: imgs[None]})[0][0] for img in imgs])
 
 
 def evaluation(
@@ -147,23 +152,44 @@ def evaluation(
     return y_true, y_pred_top_1, y_pred_top_5
 
 
+""" Decode predictions """
+
+
+def decode_predictions(preds, top=5):
+    preds = np.array(preds)
+    if preds.shape[-1] == 1000:
+        return tf.keras.applications.imagenet_utils.decode_predictions(preds, top=top)
+    elif preds.shape[-1] == 21843:
+        return decode_predictions_imagenet21k(preds, top=top)
+    else:
+        print("[Error] not imagenet or imagenet21k prediction, not supported")
+
+
 def decode_predictions_imagenet21k(preds, top=5):
     """Similar function from keras.applications.imagenet_utils.decode_predictions, just using imagenet21k class index"""
-    if len(preds.shape) != 2 or preds.shape[1] != 1000:
-        pass
-    url = "https://github.com/leondgarse/keras_cv_attention_models/releases/download/efficientdet/imagenet21k_class_index.json"
-    class_index_path = tf.keras.utils.get_file(origin=url)
-    with open(class_index_path) as ff:
-        class_index = json.load(ff)
+    if len(preds.shape) != 2 or preds.shape[-1] != 21843:
+        print("[Error] not imagenet21k prediction")
+        return
+
+    global CLASS_INDEX_IMAGENET21K
+    if CLASS_INDEX_IMAGENET21K is None:
+        url = "https://github.com/leondgarse/keras_cv_attention_models/releases/download/efficientdet/imagenet21k_class_index.json"
+        class_index_path = tf.keras.utils.get_file(origin=url)
+        with open(class_index_path) as ff:
+            CLASS_INDEX_IMAGENET21K = json.load(ff)
+
     results = []
     for pred in preds:
         top_indices = pred.argsort()[-top:][::-1]
-        result = [tuple(class_index[str(i)]) + (pred[i],) for i in top_indices]
+        result = [tuple(CLASS_INDEX_IMAGENET21K[str(i)]) + (pred[i],) for i in top_indices]
         result.sort(key=lambda x: x[2], reverse=True)
         results.append(result)
     return results
 
 
+""" Plotting related """
+
+
 def parse_timm_log(log_file, pick_keys=None):
     with open(log_file, "r") as ff:
         aa = ff.readlines()
@@ -281,8 +307,12 @@ def plot_hists(hists, names=None, base_size=6, addition_plots=["lr"], text_va=["
         if acc_key not in hist:
             all_acc_key = [ii for ii in hist.keys() if "acc" in ii and "val" not in ii]
             acc_key = "acc" if len(all_acc_key) == 0 else all_acc_key[0]
+
         val_acc_key = "val_acc"
-        if val_acc_key not in hist:
+        if "val_ap_ar" in hist:  # It's coco hist
+            hist["val_ap_0.50:0.95"] = [ii[0] for ii in hist["val_ap_ar"]]
+            val_acc_key = "val_ap 0.50:0.95"
+        elif val_acc_key not in hist:
             all_val_acc_key = [ii for ii in hist.keys() if "acc" in ii and "val" in ii]
             val_acc_key = "val_acc" if len(all_val_acc_key) == 0 else all_val_acc_key[0]
 
@@ -295,11 +325,11 @@ def plot_hists(hists, names=None, base_size=6, addition_plots=["lr"], text_va=["
 
         acc = hist.get(acc_key, [])
         if len(acc) > 0:  # For timm log
-            plot_and_peak_scatter(axes[1], acc, np.argmax, name + " accuracy", skip_first, color, cur_va, cur_pred_curve)
+            plot_and_peak_scatter(axes[1], acc, np.argmax, name + " " + acc_key, skip_first, color, cur_va, cur_pred_curve)
 
         val_acc = hist.get(val_acc_key, [])
         if len(val_acc) > 0:  # For timm log
-            plot_and_peak_scatter(axes[1], val_acc, np.argmax, name + " val_accuracy", skip_first, color, cur_va, cur_pred_curve, linestyle="--")
+            plot_and_peak_scatter(axes[1], val_acc, np.argmax, name + " " + val_acc_key, skip_first, color, cur_va, cur_pred_curve, linestyle="--")
 
         if addition_plots is not None and len(addition_plots) != 0:
             for id, ii in enumerate(addition_plots):
